#include <Arduino.h>
#include <ESP8266WiFi.h>
#include <espnow.h>

class Transmitter {
	uint8_t (*_targets)[6];
	unsigned int _targetCount;

public:
	template <size_t rows>
	Transmitter(uint8_t(&targets)[rows][6]) {
		_targets = targets;
		_targetCount = rows;
	}
	
	void setup() {
		WiFi.mode(WIFI_STA);

		if (esp_now_init() != 0) {
			Serial.println("Error initializing ESP-NOW");
			return;
		}

		esp_now_set_self_role(ESP_NOW_ROLE_CONTROLLER);
		for (unsigned int i = 0; i < _targetCount; i++) {
			Serial.print("Registering recipient ");
			Serial.print(i);
			Serial.print(": ");
			auto result = esp_now_add_peer(_targets[i], ESP_NOW_ROLE_SLAVE, 1, NULL, 0);
			Serial.println(result);
		}
	}

	void send(int value) {
		for (unsigned int i = 0; i < _targetCount; i++) {
			Serial.print("Sending to recipient ");
			Serial.print(i);
			Serial.print(": ");
			auto result = esp_now_send(_targets[i], (uint8_t *)&value, sizeof(value));
			Serial.println(result);
		}
	}
};
