#ifndef Timer_cpp
#define Timer_cpp

#include <Arduino.h>
#include <functional>

class Timer{
private:
	unsigned long _interval, _lastTick;
	std::function<void()> _callback;
	bool _running;

public:
	Timer(std::function<void()> callback, unsigned long interval) {
		_callback = callback;
		_running = false;
		_interval = interval;
	}

	void reset(){
		_lastTick = millis();
		_running = true;
	}

	void loop(){
		if (!_running)
			return;
			
		unsigned long tick = millis();

		if (tick - _lastTick >= _interval){
			_callback();
			_lastTick = millis();
		}
	}

	void start() {
		_running = true;
	}

	void stop() {
		_running = false;
	}
};

#endif
