#include <Arduino.h>
#include <ESP8266mDNS.h>
#include <ESP8266WiFi.h>
#include <WiFiUdp.h>
#include <ArduinoOTA.h>
#include <espnow.h>
#include "PWMOutput.cpp"
#include "Timer.cpp"

#define WARMWHITE_PIN 5
#define COLDWHITE_PIN 13
#define RED_PIN 4
#define GREEN_PIN 12
#define BLUE_PIN 14

IPAddress apIP(192, 168, 4, 1);
PWMOutput _warmWhite(5);
PWMOutput _coldWhite(13);
PWMOutput _red(4);
PWMOutput _green(12);
PWMOutput _blue(14);
byte _stage;
bool _partyMode;

Timer _colourTimer([] {
	switch (_stage)
	{
	case 0:
		_red.on();
		_green.off();
		_blue.off();
		break;
	case 1:
		_red.off();
		_green.on();
		_blue.off();
		break;
	case 2:
		_red.off();
		_green.off();
		_blue.on();
		break;
	default:
		_stage = 0;
		break;
	}

	_stage++;
	if (_stage > 2)
		_stage = 0;
}, 1000);

void partyOn() {
	if (_partyMode)
		return;

	_partyMode = true;
	_colourTimer.reset();
	_warmWhite.off();
	_red.on();
	_green.off();
	_blue.off();
	_stage = 1;
}

void partyOff() {
	_partyMode = false;
	_colourTimer.stop();
	_warmWhite.on();
	_red.off();
	_green.off();
	_blue.off();
}

void OnDataRecv(uint8_t* mac, uint8_t* incomingData, uint8_t len) {
	bool on;
	memcpy(&on, incomingData, sizeof(on));
	Serial.print("ESP-NOW: ");
	Serial.println(on);

	on ? partyOn() : partyOff();
}

void setupReceiver() {
	Serial.println("Init ESP-NOW");
	if (esp_now_init() != 0) {
		Serial.println("Error initialising ESP-NOW");
		return;
	}

	// esp_now_set_self_role(ESP_NOW_ROLE_SLAVE);
	esp_now_register_recv_cb(OnDataRecv);
	Serial.println("Init ESP-NOW complete");
}

void setup() {
	Serial.begin(115200);

	_warmWhite.setValue(128);
	_warmWhite.on();

	WiFi.mode(WIFI_AP_STA);
	WiFi.hostname("PartyBulb");
	WiFi.softAPConfig(apIP, apIP, IPAddress(255, 255, 255, 0));
	WiFi.softAP("PartyBulb");

	setupReceiver();

	ArduinoOTA.onStart([]() {
		Serial.println("ArduinoOTA: Start");
	});
	ArduinoOTA.onEnd([]() {
		Serial.println("\nArduinoOTA: End");
	});
	ArduinoOTA.onProgress([](unsigned int progress, unsigned int total) {
		Serial.printf("ArduinoOTA: Progress: %u%%\r", (progress / (total / 100)));
	});
	ArduinoOTA.onError([](ota_error_t error) {
		Serial.printf("Error[%u]: ", error);
		switch (error) {
		case OTA_AUTH_ERROR:
			Serial.println("ArduinoOTA: Auth Failed");
			break;
		case OTA_BEGIN_ERROR:
			Serial.println("ArduinoOTA: Begin Failed");
			break;
		case OTA_CONNECT_ERROR:
			Serial.println("ArduinoOTA: Connect Failed");
			break;
		case OTA_RECEIVE_ERROR:
			Serial.println("ArduinoOTA: Receive Failed");
			break;
		case OTA_END_ERROR:
			Serial.println("ArduinoOTA: End Failed");
			break;
		default:
			break;
		}
	});
	ArduinoOTA.begin();

	Serial.println("Setup end");
}

void loop() {
	ArduinoOTA.handle();
	_colourTimer.loop();
	delay(10);
}
