#include <Arduino.h>
#include "SoftwareSerial.h"
#include "DFRobotDFPlayerMini.h"
#include <ESP8266WiFi.h>
#include <espnow.h>

SoftwareSerial _softwareSerial(2, 0); // RX, TX
DFRobotDFPlayerMini _dfPlayer;
bool _playing = false;

void play() {
	if (_playing)
		return;

	Serial.println("Playing");
	_playing = true;
	_dfPlayer.randomAll();
}

void stop() {
	if (!_playing)
		return;
	
	_playing = false;
	Serial.println("Stopping");
	_dfPlayer.stop();
}

void OnDataRecv(uint8_t* mac, uint8_t* incomingData, uint8_t len) {
	int playing;
	memcpy(&playing, incomingData, sizeof(playing));
	Serial.println(playing);

	playing ? play() : stop();
}

void setupReceiver() {
	WiFi.mode(WIFI_STA);

	if (esp_now_init() != 0) {
		Serial.println("Error initialising ESP-NOW");
		return;
	}

	esp_now_set_self_role(ESP_NOW_ROLE_SLAVE);
	esp_now_register_recv_cb(OnDataRecv);
}

void handleState(uint8_t type, int value) {
	switch (type) {
	case TimeOut:
		Serial.println(F("Time Out!"));
		break;
	case WrongStack:
		Serial.println(F("Stack Wrong!"));
		break;
	case DFPlayerCardInserted:
		Serial.println(F("Card Inserted!"));
		break;
	case DFPlayerCardRemoved:
		Serial.println(F("Card Removed!"));
		break;
	case DFPlayerCardOnline:
		Serial.println(F("Card Online!"));
		break;
	case DFPlayerPlayFinished:
		Serial.print(F("Number:"));
		Serial.print(value);
		Serial.println(F(" Play Finished!"));
		break;
	case DFPlayerError:
		Serial.print(F("DFPlayerError:"));
		switch (value) {
		case Busy:
			Serial.println(F("Card not found"));
			break;
		case Sleeping:
			Serial.println(F("Sleeping"));
			break;
		case SerialWrongStack:
			Serial.println(F("Get Wrong Stack"));
			break;
		case CheckSumNotMatch:
			Serial.println(F("Check Sum Not Match"));
			break;
		case FileIndexOut:
			Serial.println(F("File Index Out of Bound"));
			break;
		case FileMismatch:
			Serial.println(F("Cannot Find File"));
			break;
		case Advertise:
			Serial.println(F("In Advertise"));
			break;
		default:
			break;
		}
		break;
	default:
		break;
	}
}

void setup() {
	_softwareSerial.begin(9600);
	Serial.begin(115200);
	setupReceiver();

	Serial.println();
	Serial.println(F("DFRobot DFPlayer Mini Demo"));
	Serial.println(F("Initializing DFPlayer ... (May take 3~5 seconds)"));

	if (!_dfPlayer.begin(_softwareSerial)) {
		Serial.println(F("Unable to begin:"));
		Serial.println(F("1.Please recheck the connection!"));
		Serial.println(F("2.Please insert the SD card!"));
		while (true);
	}
	Serial.println(F("DFPlayer Mini online."));

	_dfPlayer.volume(30); // Max 30
}

void loop() {
	if (_dfPlayer.available()) {
		handleState(_dfPlayer.readType(), _dfPlayer.read());
	}
}